///////////////////////////////////////////////////////////////////////////////
//
// Module: input_small_fifo.v
// Project: BJP1
// Description: 
//              In input arbitter, when a packet received, it's directly sent
//				to this FIFO buffer word by word. And at the first received byte
//				of packet, buffer records received time_stamp and time_slot.
//
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns/1ps
  
  module input_small_fifo
    #(parameter WIDTH = 72,
	  parameter CTRL_WIDTH = 8,
      parameter MAX_DEPTH_BITS = 3,
      parameter NEARLY_FULL = 2**MAX_DEPTH_BITS - 3,
	  parameter MAIN_TIME_BITS=19)
    (
	//to mac	   
     input [WIDTH-1:0] din,     // Data in
     input          wr_en,   // Write enable
     output         not_nearly_full,
     
	 // to arbiter
	 output reg [WIDTH-1:0]  dout,    // Data out
     input          rd_en,   // Read the next word 
     output reg          empty,
	 
	 output reg  pkt_rcv_slot_no,
	 output reg [MAIN_TIME_BITS-1:0] pkt_rcv_tm_stmp,
	 
	 //timing wires
     input          reset,
     input          clk,
     input  [MAIN_TIME_BITS-1:0]      time_stamp,
     input          slot_no_lsb,
	 output reg ptr
	 );



parameter MAX_DEPTH        = 2 ** MAX_DEPTH_BITS;
   
reg [WIDTH-1:0] queue [MAX_DEPTH - 1 : 0];
reg [MAX_DEPTH_BITS - 1 : 0] rd_ptr;
reg [MAX_DEPTH_BITS - 1 : 0] wr_ptr;
reg [MAX_DEPTH_BITS -1 : 0] depth;



always @(posedge clk)
begin
   if (wr_en) 
      queue[wr_ptr] <= din;
	if (wr_en && (din [WIDTH-1:WIDTH-CTRL_WIDTH] == 'hff)) 
		begin
		pkt_rcv_slot_no <= slot_no_lsb;
		pkt_rcv_tm_stmp <= time_stamp;
		end
	if (rd_en && !empty)
      dout <= queue[rd_ptr];
end
always @(posedge clk) 
begin
if (reset) 	  ptr <= 1;
else begin if (rd_en && !empty)
	  ptr <= rd_ptr[0];
  end
end

always @(posedge clk)
begin
   if (reset) begin
      rd_ptr <= 'h0;
      wr_ptr <= 'h0;
      depth  <= 'h0;
   end
   else begin
      if (wr_en) wr_ptr <= wr_ptr + 'h1;
      if (rd_en && !empty) rd_ptr <= rd_ptr + 'h1;
      if (wr_en & ~rd_en) depth <= depth + 'h1;
      else if (~wr_en & rd_en & (depth!=0)) depth <= depth - 'h1;
   end
end

always @(posedge clk) begin
	empty <= (wr_ptr== rd_ptr+(rd_en & !empty)) && !depth[MAX_DEPTH_BITS - 1];
end
assign not_nearly_full = depth < NEARLY_FULL;


endmodule 
