`timescale 1ns/1ps


  module my_input_arbiter
    #(parameter DATA_WIDTH = 64,
      parameter CTRL_WIDTH=DATA_WIDTH/8,
	  parameter MAIN_TIME_BITS=19,
	  parameter SRAM_DATA_WIDTH = DATA_WIDTH+CTRL_WIDTH,
      parameter SRAM_ADDR_WIDTH = 19,
	  parameter UDP_REG_SRC_WIDTH = 2,
	  parameter LAG_LENGTH = 5,
	  parameter ROUTE_LENGTH = 4,
	  parameter INTER_MOD_WIDTH = SRAM_ADDR_WIDTH + LAG_LENGTH + 1,
	  parameter PACKET_ID_LENGTH = 12,
	  parameter NO_OF_PORTS=4,
	  parameter PORT_BITS = 2,
	  parameter EVENT_SIZE = PACKET_ID_LENGTH+MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS
      )

   (// From MACs
    input  [DATA_WIDTH-1:0]            in_data_0,
    input  [CTRL_WIDTH-1:0]            in_ctrl_0,
    input                              in_wr_0,
    output                             in_rdy_0,

    input  [DATA_WIDTH-1:0]            in_data_2,
    input  [CTRL_WIDTH-1:0]            in_ctrl_2,
    input                              in_wr_2,
    output                             in_rdy_2,

	input  [DATA_WIDTH-1:0]            in_data_4,
    input  [CTRL_WIDTH-1:0]            in_ctrl_4,
    input                              in_wr_4,
    output                             in_rdy_4,
	
	input  [DATA_WIDTH-1:0]            in_data_6,
    input  [CTRL_WIDTH-1:0]            in_ctrl_6,
    input                              in_wr_6,
    output                             in_rdy_6,
   // To Scheduler
    output reg [INTER_MOD_WIDTH-1:0]        sch,
    output reg [NO_OF_PORTS-1:0]                        sch_wr,	

	
	// To Sram
	output reg [SRAM_ADDR_WIDTH-1:0]       wr_0_addr,
    output reg  [SRAM_DATA_WIDTH-1:0]       wr_0_data,
    output reg                            wr_0_req,
    input                              wr_0_ack,
	
	// To Logger
	output  reg [EVENT_SIZE-1:0]			event0,
	output 	reg						event_wr,	
	//register
    input                                  reg_req_in,
    input                                  reg_ack_in,
    input                                  reg_rd_wr_L_in,
    input  [`UDP_REG_ADDR_WIDTH-1:0]       reg_addr_in,
    input  [`CPCI_NF2_DATA_WIDTH-1:0]      reg_data_in,
    input  [UDP_REG_SRC_WIDTH-1:0]         reg_src_in,

    output                             reg_req_out,
    output                             reg_ack_out,
    output                             reg_rd_wr_L_out,
    output [`UDP_REG_ADDR_WIDTH-1:0]   reg_addr_out,
    output [`CPCI_NF2_DATA_WIDTH-1:0]  reg_data_out,
    output [UDP_REG_SRC_WIDTH-1:0]     reg_src_out,
	
    //Timings
    input                              reset,
    input                              clk,
	input  [MAIN_TIME_BITS-1:0]      time_stamp,
    input      				    slot_no
	 
   );
//-----------------------input_fifo s---------------------------
   wire [CTRL_WIDTH-1:0]               fifo_out_ctrl[3:0];
   wire [DATA_WIDTH-1:0]               fifo_out_data[3:0];
   reg [NO_OF_PORTS-1:0]                rd_en;
   wire [NO_OF_PORTS-1:0]               empty;

   wire [NO_OF_PORTS-1:0]				pkt_slot_no ;
   wire [MAIN_TIME_BITS-1:0]			pkt_tm_stmp [NO_OF_PORTS-1:0];
   wire [NO_OF_PORTS-1:0] ptr;

input_small_fifo
    #(.WIDTH(SRAM_DATA_WIDTH),
	  .CTRL_WIDTH(CTRL_WIDTH),
      .MAX_DEPTH_BITS(3),
      .NEARLY_FULL(2**3 - 3),
	  .MAIN_TIME_BITS(MAIN_TIME_BITS))
  input_small_fifo_0
    (.din({in_ctrl_0, in_data_0}),
     .wr_en(in_wr_0),
     .not_nearly_full(in_rdy_0),

     .dout({fifo_out_ctrl[0], fifo_out_data[0]}), 
     .rd_en(rd_en[0]),
     .empty(empty[0]),
	 
     .pkt_rcv_slot_no(pkt_slot_no[0]),
	 .pkt_rcv_tm_stmp(pkt_tm_stmp[0]),
     
     .reset(reset),
     .clk(clk),
	 .time_stamp(time_stamp),
     .slot_no_lsb(slot_no),
	 .ptr(ptr[0])
     );

	 
input_small_fifo
    #(.WIDTH(SRAM_DATA_WIDTH),
	  .CTRL_WIDTH(CTRL_WIDTH),
      .MAX_DEPTH_BITS(3),
      .NEARLY_FULL(2**3 - 3),
	  .MAIN_TIME_BITS(MAIN_TIME_BITS))
  input_small_fifo_1
    (.din({in_ctrl_2, in_data_2}),
     .wr_en(in_wr_2),
     .not_nearly_full(in_rdy_2),

     .dout({fifo_out_ctrl[1], fifo_out_data[1]}), 
     .rd_en(rd_en[1]),
     .empty(empty[1]),
	 
     .pkt_rcv_slot_no(pkt_slot_no[1]),
	 .pkt_rcv_tm_stmp(pkt_tm_stmp[1]),
     
	 .reset(reset),
     .clk(clk),
	 .time_stamp(time_stamp),
     .slot_no_lsb(slot_no),
	 .ptr(ptr[1])
     );	 
	   

input_small_fifo
    #(.WIDTH(SRAM_DATA_WIDTH),
	  .CTRL_WIDTH(CTRL_WIDTH),
      .MAX_DEPTH_BITS(3),
      .NEARLY_FULL(2**3 - 3),
	  .MAIN_TIME_BITS(MAIN_TIME_BITS))
	  input_small_fifo_2
    (.din({in_ctrl_4, in_data_4}),
     .wr_en(in_wr_4),
     .not_nearly_full(in_rdy_4),

     .dout({fifo_out_ctrl[2], fifo_out_data[2]}), 
     .rd_en(rd_en[2]),
     .empty(empty[2]),
	 
     .pkt_rcv_slot_no(pkt_slot_no[2]),
	 .pkt_rcv_tm_stmp(pkt_tm_stmp[2]),
     
	 .reset(reset),
     .clk(clk),
	 .time_stamp(time_stamp),
     .slot_no_lsb(slot_no),
	 .ptr(ptr[2])
     );	 
	 
	 
	 
input_small_fifo
    #(.WIDTH(SRAM_DATA_WIDTH),
	  .CTRL_WIDTH(CTRL_WIDTH),
      .MAX_DEPTH_BITS(3),
      .NEARLY_FULL(2**3 - 3),
	  .MAIN_TIME_BITS(MAIN_TIME_BITS))
  input_small_fifo_3
    (.din({in_ctrl_6, in_data_6}),
     .wr_en(in_wr_6),
     .not_nearly_full(in_rdy_6),

     .dout({fifo_out_ctrl[3], fifo_out_data[3]}), 
     .rd_en(rd_en[3]),
     .empty(empty[3]),
	 
     .pkt_rcv_slot_no(pkt_slot_no[3]),
	 .pkt_rcv_tm_stmp(pkt_tm_stmp[3]),
     
	 .reset(reset),
     .clk(clk),
	 .time_stamp(time_stamp),
     .slot_no_lsb(slot_no),
	 .ptr(ptr[3])
     );	 

//----------------------sram_buffer-----------------------   
wire bfr_nearly_full;
   
reg [DATA_WIDTH+CTRL_WIDTH+SRAM_ADDR_WIDTH-1:0]	sram_bfr_din;   
reg						bfr_wr_en;
wire [DATA_WIDTH+CTRL_WIDTH+SRAM_ADDR_WIDTH-1:0]	sram_bfr_dout;
wire 							bfr_empty;
reg							bfr_rd_en;

reg [DATA_WIDTH+CTRL_WIDTH-1:0] sram_data;
reg [SRAM_ADDR_WIDTH-1:0] sram_addr;
   
reg [1:0] q_state;   
   
    my_small_fifo
    #(.WIDTH(SRAM_DATA_WIDTH+SRAM_ADDR_WIDTH),
      .MAX_DEPTH_BITS(3),
      .NEARLY_FULL(4))
    my_small_fifo
    (   
     .din				(sram_bfr_din),
     .wr_en				(bfr_wr_en),
     .rd_en				(bfr_rd_en), 
     
     .dout    			(sram_bfr_dout),
     .nearly_full		(bfr_nearly_full),
     .empty				(bfr_empty),
     
     .reset				(reset),
     .clk				(clk)
    );

	
	
//------------------------------------regs----------------------------------------
wire [15:0] lu_table0;
wire [15:0] lu_table1;
wire [15:0] lu_table2;
wire [15:0] lu_table3;
my_in_arb_regs
#(
  .REG_START_ADDR(`ROUTER_OP_LUT_ARP_MAC_HI_REG)
) my_in_arb_regs ( 
  .reg_req_in(reg_req_in),
  .reg_ack_in(reg_ack_in),
  .reg_rd_wr_L_in(reg_rd_wr_L_in),
  .reg_addr_in(reg_addr_in),
  .reg_data_in(reg_data_in),
  .reg_src_in(reg_src_in),

  .reg_req_out(reg_req_out),
  .reg_ack_out(reg_ack_out),
  .reg_rd_wr_L_out(reg_rd_wr_L_out),
  .reg_addr_out(reg_addr_out),
  .reg_data_out(reg_data_out),
  .reg_src_out(reg_src_out),

  // --- SW regs interface
  .software_regs({lu_table3, lu_table2, lu_table1, lu_table0}), // signals from the software
  
  .clk              (clk),
  .reset            (reset)
);

//N ------------------------------write to sram--------------------------------------
reg state;
reg state_next;
reg wr_0_req_next;
reg [SRAM_ADDR_WIDTH-1:0] wr_0_addr_next;
reg [SRAM_DATA_WIDTH-1:0] wr_0_data_next; 

   parameter IDLE = 0;
   parameter WRITE = 1;
 always @(posedge clk) begin
	if(reset) begin
		state <= IDLE;
		wr_0_req <= 0;
		wr_0_data <= 0;
		wr_0_addr <= 0;
	end else begin
		
		state <= state_next;
		wr_0_data <= wr_0_data_next;
		wr_0_addr <= wr_0_addr_next;
		wr_0_req <= wr_0_req_next;
end
end

always @* begin

	wr_0_data_next = wr_0_data;
	wr_0_addr_next = wr_0_addr;
	wr_0_req_next = wr_0_req;
	state_next = state;
	
	bfr_rd_en = 0;
	case(state)
		IDLE: begin
			wr_0_req_next = 0;
			if(!bfr_empty) begin
				state_next = WRITE;
				bfr_rd_en = 1;
			end       
		end 
		WRITE: begin
			wr_0_req_next = 1;
			if (!wr_0_req | wr_0_ack) begin
				wr_0_data_next = sram_bfr_dout[SRAM_DATA_WIDTH+SRAM_ADDR_WIDTH-1:SRAM_ADDR_WIDTH];
				wr_0_addr_next = sram_bfr_dout[SRAM_ADDR_WIDTH-1:0];
			end
			if(wr_0_ack & !bfr_empty) begin
				bfr_rd_en = 1;
			end
			else if (wr_0_ack) begin
				state_next = IDLE;
			end
		end 
		default: begin end
	endcase
end  


//--------------------------------read from input fifos;write in sram fifo----------------------------- 


reg [PORT_BITS-1:0] cur_q;
reg [PORT_BITS-1:0] next_q;
reg [PORT_BITS-1:0] next_q2;

reg [DATA_WIDTH-1:0] rec_data;
reg [CTRL_WIDTH-1:0] rec_ctrl;
reg [SRAM_ADDR_WIDTH-1:0]       pkt_addr [NO_OF_PORTS-1:0];
reg new_data;
reg [NO_OF_PORTS-1:0] read_empty;
reg read_on_next_clock;

wire [NO_OF_PORTS-1:0] kk;
assign kk =  (empty & read_empty);
reg [1:0] add;

wire [2:0]pluse1;
wire [2:0]pluse2;
wire [2:0]pluse3;

assign pluse1=next_q2+1;
assign pluse2=next_q2+2;
assign pluse3=next_q2+3;


always @(posedge clk) begin
	if (reset) begin
        next_q2 <= 0;

		add = 0;
		read_on_next_clock <= 0;	
		rd_en='b0000;
		read_empty <= 0;
	end else begin   
        
		
		//----choosing nextQ-----
		add[0] = !kk[pluse1[1:0]] | (!kk[pluse3[1:0]] & kk[pluse2[1:0]]);
		add[1] = (kk[pluse1[1:0]] & !kk[pluse2[1:0]]) | (kk[pluse1[1:0]] & !kk[pluse3[1:0]]);
		next_q2[1:0] <= next_q2 + add;
		
		rd_en = 'b0000;
		read_on_next_clock <= 0;
        if ((!empty[next_q2] || !read_empty[next_q2]) && !bfr_nearly_full)begin
                read_empty[next_q2] <= empty[next_q2];
				read_on_next_clock <= 1;
                next_q <= next_q2;
                rd_en[next_q2] = 1;
        end 

end
end

reg  [3:0] ptr_p ;

always @(posedge clk)
begin
	if(reset)begin
		cur_q <= 0;
		pkt_addr[0] <= 'b0000000000000000000;
		pkt_addr[1] <= 'b0100000000000000000;
		pkt_addr[2] <= 'b1000000000000000000;
		pkt_addr[3] <= 'b1100000000000000000;
		
		rec_ctrl <= 0;
		bfr_wr_en <= 0;
		new_data <= 0;
		ptr_p <= 0;

	end
	else begin
		new_data <= 0;
	    if (read_on_next_clock)begin
                rec_data <= fifo_out_data[next_q];
                rec_ctrl <= fifo_out_ctrl[next_q];
				cur_q <= next_q;
				new_data <=ptr[next_q] ^ ptr_p[next_q];
				ptr_p[next_q] <= ptr[next_q];
        end
	pkt_addr[cur_q][SRAM_ADDR_WIDTH-PORT_BITS-1:0] <= pkt_addr[cur_q][SRAM_ADDR_WIDTH-PORT_BITS-1:0]+(new_data);
	bfr_wr_en <= new_data;
	sram_bfr_din <= {rec_ctrl, rec_data, pkt_addr[cur_q] };
end
end	




//--------------------------------event sch---------------------------

reg [3:0] new_pkt;

always @(posedge clk)
begin

if (new_data) new_pkt[cur_q] <= rec_ctrl =='hff;

event_wr <= 0;
if (new_data && new_pkt[cur_q]) begin
/////  EVENT = PACKET_ID || TIME || SRAM ADDR || LENGTH || LAG || PORT
	event0[PORT_BITS-1:0] <= cur_q;
	event0[LAG_LENGTH+ROUTE_LENGTH+PORT_BITS-1:PORT_BITS] <= rec_data[DATA_WIDTH-1:DATA_WIDTH-LAG_LENGTH-ROUTE_LENGTH];
	event0[SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS-1:LAG_LENGTH+ROUTE_LENGTH+PORT_BITS] <= pkt_addr[cur_q]-1;
	event0[MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS-1:SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS] <= pkt_tm_stmp[cur_q];
	event0[PACKET_ID_LENGTH+MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS-1:MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS] <= rec_data[PACKET_ID_LENGTH-1:0];
	event_wr <= 1;
end
end

//--------------------------------------routing algo----------------------------------
always @(posedge clk) begin
sch_wr <= 0;
 if (new_data && new_pkt[cur_q]) begin
	sch_wr[0] <= |rec_data[DATA_WIDTH-1:DATA_WIDTH-ROUTE_LENGTH];//lu_table0[rec_data[63:60]];//(rec_data[63:60]===0);
	sch_wr[1] <= !(|rec_data[DATA_WIDTH-1:DATA_WIDTH-ROUTE_LENGTH]);//lu_table1[rec_data[63:60]];//(rec_data[63:60]===1);
	//sch_wr[2] <= lu_table2[rec_data[63:60]];//(rec_data[63:60]===2);
	//sch_wr[3] <= lu_table3[rec_data[63:60]];//(rec_data[63:60]===3);
	sch[LAG_LENGTH:1] <= rec_data[DATA_WIDTH-ROUTE_LENGTH-1:DATA_WIDTH-ROUTE_LENGTH-LAG_LENGTH];
end
sch[0] <= pkt_slot_no[cur_q];
sch[INTER_MOD_WIDTH-1:INTER_MOD_WIDTH-SRAM_ADDR_WIDTH] <= pkt_addr[cur_q]-1;

end
endmodule

