///////////////////////////////////////////////////////////////////////////////
//
// Module: my_small_fifo.v
// Project: BJP1
// Description: 
//				It's a multi-propose FIFO, for getting data from previous module,
//				writing and reading to SRAM and etc.
//
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

  module my_small_fifo
    #(parameter WIDTH = 91,
      parameter MAX_DEPTH_BITS = 3,
      parameter NEARLY_FULL = 2**MAX_DEPTH_BITS - 1)
    (
	 // Data IN	   
     input [WIDTH-1:0] din,
     input          wr_en,
     input          rd_en,
     // Data OUT
     output reg [WIDTH-1:0]  dout,
     output         nearly_full,
     output reg        empty,
     //Timing
     input          reset,
     input          clk
     );


parameter MAX_DEPTH        = 2 ** MAX_DEPTH_BITS;
   
reg [WIDTH-1:0] queue [MAX_DEPTH - 1 : 0];
reg [MAX_DEPTH_BITS - 1 : 0] rd_ptr;
reg [MAX_DEPTH_BITS - 1 : 0] wr_ptr;
reg [MAX_DEPTH_BITS - 1 : 0] depth;


always @(posedge clk)
begin
   if (wr_en) 
      queue[wr_ptr] <= din;
end

always @(posedge clk)
begin
   if (reset) begin
      rd_ptr <= 'h0;
      wr_ptr <= 'h0;
      depth  <= 'h0;
   end
   else begin
      if (wr_en) wr_ptr <= wr_ptr + 'h1;
      if (rd_en && !empty) rd_ptr <= rd_ptr + 'h1;
      if (wr_en & ~rd_en ) depth <= depth + 'h1;
      else if (~wr_en & rd_en  & (depth!=0)) depth <= depth - 'h1;
   end
end
always @(posedge clk)
begin
	if (rd_en && !empty)
      dout <= queue[rd_ptr];
end

assign nearly_full = depth >= NEARLY_FULL;
always @(posedge clk) begin
	empty <= (wr_ptr== rd_ptr+(rd_en & !empty)) && !depth[MAX_DEPTH_BITS - 1];
end

endmodule 
