///////////////////////////////////////////////////////////////////////////////
//
// Module: my_sram_read_arbiter.v
// Project: BJP1
// Description: 
//              
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////


`timescale 1ns/1ps

module my_sram_read_arbiter
#( parameter DATA_WIDTH = 64,
	  parameter CTRL_WIDTH=DATA_WIDTH/8,
	  parameter SRAM_ADDR_WIDTH = 19,
	  parameter SRAM_DATA_WIDTH = DATA_WIDTH+CTRL_WIDTH,
	  parameter SENDERS_NO = 4
	)

	( 
	//To SRAM
    input  [SRAM_DATA_WIDTH-1:0]       rd_0_data,
    input                              rd_0_vld,

	input                              rd_0_ack,
    output [SRAM_ADDR_WIDTH-1:0]       rd_0_addr,
    output                             rd_0_req,
	
	//From Senders SRAM
    output reg  [SRAM_DATA_WIDTH-1:0]       data_out,
    output reg  [SENDERS_NO-1:0]                  wr,
///////////
	output reg [SENDERS_NO-1:0]             ack,
	input [SENDERS_NO-1:0]             req,	
    
	input [SRAM_ADDR_WIDTH-1:0]      addr0,
	input [SRAM_ADDR_WIDTH-1:0]      addr1,
	input [SRAM_ADDR_WIDTH-1:0]      addr2,
	input [SRAM_ADDR_WIDTH-1:0]      addr3,	
	//Timings
	input                              reset,
    input                              clk

	);


wire     [SRAM_ADDR_WIDTH-1:0]  addr [SENDERS_NO-1:0] ;
assign addr[0] = addr0;
assign addr[1] = addr1;
assign addr[2] = addr2;
assign addr[3] = addr3;
	
reg [SRAM_ADDR_WIDTH+1:0] bfr_in_data;
reg bfr_wr_en;

wire bfr_nearly_full;


reg [1:0] serve;
reg [1:0] add;

reg [1:0]pluse1;
reg [1:0]pluse2;
reg [1:0]pluse3;

always @* begin
	pluse1=serve+1;
	pluse2=serve+2;
	pluse3=serve+3;
end

always @(posedge clk) begin
if (reset) begin
	add = 0;
	serve <= 0;
end else begin
	add[0] = req[pluse1[1:0]] | (req[pluse3[1:0]] & !req[pluse2[1:0]]);
	add[1] = !req[pluse1[1:0]] & (req[pluse2[1:0]] | req[pluse3[1:0]]);
	serve[1:0] <= serve + add;

end
end

always @(posedge clk) begin

	bfr_in_data[SRAM_ADDR_WIDTH-1:0] = addr[serve];
	bfr_in_data[SRAM_ADDR_WIDTH+1:SRAM_ADDR_WIDTH] = serve;
	
	bfr_wr_en = 0;
	if (!bfr_nearly_full)
	begin
		bfr_wr_en = req[serve];
	end

end

always @* begin
	ack = 0;
	if (!bfr_nearly_full)
	begin
	ack[serve] = req[serve];
	end

end

wire [1:0] which_input;
sram_arbiter_fifo
    #( .WIDTH(SRAM_ADDR_WIDTH+2),
       .MAX_DEPTH_BITS(3),
       .NEARLY_FULL(7))
 sram_arbiter_fifo
    ( 
     .din				(bfr_in_data),
     .wr_en				(bfr_wr_en),
     .rd_en				(rd_0_ack), 
     
     .dout    			({which_input, rd_0_addr}),
	 .nearly_full		(bfr_nearly_full),
     .not_empty				(rd_0_req),
     
     .reset				(reset),
     .clk				(clk)
    );


reg [1:0] wi_memory [8:0];
reg [2:0] wi_wr_ptr;
reg [2:0] wi_rd_ptr;
reg [2:0] wi_rd_ptr_new;
reg [1:0] f;

always @(posedge clk) begin	
if (reset) begin
	wi_wr_ptr <= 0;
	wi_rd_ptr <= 0;
	f <= 0;
end
else begin
	if(rd_0_ack)
		wi_wr_ptr <= wi_wr_ptr+1;
	if(rd_0_req) 
		wi_memory[wi_wr_ptr] = which_input;

	
	wi_rd_ptr <= wi_rd_ptr_new;
	if(rd_0_vld) begin
		f <= wi_memory[wi_rd_ptr];
	end
end	
end
always @(*) begin
	if (reset)
		wi_rd_ptr_new = 0;
	else
		if(rd_0_vld)
			wi_rd_ptr_new = wi_rd_ptr+1;
end



reg wr_on_next;
reg [SRAM_DATA_WIDTH-1:0] wr_data;
always @(posedge clk) begin
if(reset) begin
	wr_on_next <= 0;
	wr <= 0;

end else begin
	wr_on_next <= 0;
	if(rd_0_vld) begin
		wr_on_next <= 1;
		wr_data <= rd_0_data;
	end
	
	wr <= 0;
	if (wr_on_next) begin
		data_out <= wr_data;
		wr[f] <= 1;
	end
	
end
end	
	
	
	
	
endmodule 