///////////////////////////////////////////////////////////////////////////////
//
// Module: my_very_small_fifo.v
// Project: BJP1
// Description: 
//				It's a multi-propose FIFO, for getting data from previous module,
//				writing and reading to SRAM and etc.
//
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

  module my_very_small_fifo
    #(parameter WIDTH = 91,
      parameter MAX_DEPTH_BITS = 3)
    (
	 // Data IN	   
     input [WIDTH-1:0] din,
     input          wr_en,
     input          rd_en,
     // Data OUT
     output reg [WIDTH-1:0]  dout,
     output reg        empty,
     //Timing
     input          reset,
     input          clk
     );


parameter MAX_DEPTH        = 2 ** MAX_DEPTH_BITS;
   
reg [WIDTH-1:0] queue [MAX_DEPTH - 1 : 0];
reg [MAX_DEPTH_BITS - 1 : 0] rd_ptr;
reg [MAX_DEPTH_BITS - 1 : 0] wr_ptr;


always @(posedge clk)
begin
   if (wr_en) 
      queue[wr_ptr] <= din;
end

always @(posedge clk)
begin
   if (reset) begin
      rd_ptr <= 'h0;
      wr_ptr <= 'h0;
   end
   else begin
      if (wr_en) wr_ptr <= wr_ptr + 'h1;
      if (rd_en) rd_ptr <= rd_ptr + 'h1;
	dout = queue[rd_ptr + rd_en];
     empty= (wr_ptr == rd_ptr+rd_en);
end
end


endmodule 
