///////////////////////////////////////////////////////////////////////////////
//
// Module: scheduler.v
// Project: BJP1
// Description: 
//              This modules receive a packet_identifier* for each received
//				packet at input arbiter. Then schedule the time for sending it
//				in a table, and if the table is full, drop it.
//
//				This module divieded in 3 parts: 1- receive packet_identifier
//												 2- put it in the table
//												 3- send table entery in 
//													apropriate time
//				
//				@posedge of "near_to_change_slot" (about 50 clock cycle before
//				slot changing) Scheduler sends packet_identifier* to sender
//				module, to prepare this packet for sending on next slot.
//				
//				*: packet_identifier: a 48 bit width entery, contains start of 
//				   packet pointer in SRAM, received time_slot, and packet_ID**.
//				**:packet_ID is a 8 bit ID wich set to each packet on a round.
//
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////

module scheduler
#(	  parameter LAG_LENGTH = 5,
	parameter ROUTE_LENGTH = 4,
	parameter SRAM_ADDR_WIDTH = 19,
	parameter INTER_MOD_WIDTH = SRAM_ADDR_WIDTH + LAG_LENGTH + 1,
	parameter NORMAL_DELAY =10, //normal delay for each packet is 10 slot
	parameter TABLE_SIZE=32,												//---------------------------------------------------------------------------
	parameter PTR_SIZE=LAG_LENGTH, //size of PTR to manage the table (in bits)//-------------------------------------------------------------------
	parameter SLOT_NO_BITS=8)
        (
        // From Input Arbiter
    input  [INTER_MOD_WIDTH-1:0]      in_data,
    input                          in_wr,

        // To Sender
        output  reg [INTER_MOD_WIDTH-1:0]       out_data,
    output reg                        out_wr,

        //Timings
        input                              reset,
    input                              clk,
    input  [SLOT_NO_BITS-1:0]        slot_no,
        input                                                   near_to_change_slot
        );


//TABLE
reg [INTER_MOD_WIDTH-1:0]               packet_table    [TABLE_SIZE-1:0];
reg [TABLE_SIZE-1:0]            table_full;
//Pointers
reg [PTR_SIZE-1:0]              ptr;



//------------------input--------------
wire bfr_empty;
wire [INTER_MOD_WIDTH-1:0] bfr_dout;
reg bfr_rd_en;
  my_very_small_fifo
    #(.WIDTH(INTER_MOD_WIDTH),
      .MAX_DEPTH_BITS(2))
	  my_very_small_fifo
    (
	 // Data IN	   
     .din (in_data),
     .wr_en (in_wr),
     .rd_en (bfr_rd_en),
     // Data OUT
     .dout (bfr_dout),
     .empty (bfr_empty),
     //Timing
     .reset (reset),
     .clk (clk)
     );




//-----------------find_place----------

reg near_to_change_slot_pre;

reg f;
reg [INTER_MOD_WIDTH-1:0]               din;
reg serve_this;
reg [PTR_SIZE-1:0] counter00;
reg [PTR_SIZE-1:0] counter0;
reg [PTR_SIZE-1:0] counter1;
reg [PTR_SIZE-1:0] counter2;
reg [LAG_LENGTH-1:0] lag;
reg [1:0] set_counter;


always @(posedge clk) begin
    if(reset) begin
		table_full <= 32'b0;
		serve_this <= 0;
		bfr_rd_en<=0;
		f<= 0;
    end else begin
		
		
		near_to_change_slot_pre<=near_to_change_slot;
		out_wr<=0;
		if(!near_to_change_slot_pre && near_to_change_slot) begin
			table_full[ptr] <= 0;
			out_wr <= table_full[ptr];
		end//end else begin if(out_wr) out_wr<=0; end

		if (serve_this) begin
			bfr_rd_en <= 0;
			case (set_counter)
				0: begin 
					counter1<=counter0+f;
					set_counter<=1;
				end
				1: begin
					counter2<=counter1+din[LAG_LENGTH:1];
					set_counter<=2;
					end
				2: begin
					
					if (!table_full[counter2]) begin
						serve_this <= 0;
						packet_table [counter2] <= {din[INTER_MOD_WIDTH-1:INTER_MOD_WIDTH-SRAM_ADDR_WIDTH], lag, din[0]};
						table_full[counter2] <= 1;
					end
					else begin
						lag<=lag+1;
						counter2 <= counter2-1;
						counter00<=counter00-1;
						if (counter00==1) serve_this <=0;
					end
					end
				default: begin end
			endcase
			
		end else if (!bfr_empty) begin 
			serve_this<=1; 
			set_counter<=0;
			lag<=0;
			counter0 <=ptr+NORMAL_DELAY;
			f<=slot_no[0] ^ bfr_dout[0];
			counter00 <=NORMAL_DELAY ;
			bfr_rd_en <= 1;
			din<=bfr_dout;
		end

	end		
end

//-------------------send2sender-------------

reg slot_no_pre;

always @(posedge clk) begin
if(reset) ptr = 0;

else begin
	slot_no_pre<=slot_no[0];
	if (slot_no_pre!=slot_no[0]) ptr=ptr+1;
end
end


always @(posedge clk)
begin
        out_data <= packet_table [ptr] ;
end
endmodule

