///////////////////////////////////////////////////////////////////////////////
//
// Module: sender.v
// Project: BJP1
// Description: 
//              This module, receive packet identifier from scheduler module,
//				(on the posedge clk after posedge near_to_change_slot), and 
//				starting to read packet's data from SRAM to a FIFO. And on the
//				next slot, send it through MAC.
//				For each packet sending, it sends a log entery to logger module.
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns/1ps

module sender
	#( parameter DATA_WIDTH = 64,
	  parameter CTRL_WIDTH=DATA_WIDTH/8,
	  parameter SRAM_ADDR_WIDTH = 19,
	  parameter LAG_LENGTH = 5,
	  parameter ROUTE_LENGTH = 4,
	  parameter INTER_MOD_WIDTH = SRAM_ADDR_WIDTH + LAG_LENGTH + 1,
	  parameter MAIN_TIME_BITS=19,
	  parameter SRAM_DATA_WIDTH = DATA_WIDTH+CTRL_WIDTH,
	  parameter EVENT_SIZE = 64,
	  parameter PORT_BITS = 2,
	  parameter PACKET_ID_LENGTH = 12,
	  parameter PORT_TYPE = 2'b10
	)
	( 
	// From Scheduler
    input  [INTER_MOD_WIDTH-1:0]      in_data,
    input                          in_wr,
	
	// To MAC
    output reg [DATA_WIDTH+CTRL_WIDTH-1:0]           out_data_mac,
    output reg                          out_wr_mac,
    input                             out_rdy_mac,
	
	//From SRAM
    input  [SRAM_DATA_WIDTH-1:0]       rd_0_data,
    input                              rd_0_vld,

	input                              rd_0_ack,
    output  reg [SRAM_ADDR_WIDTH-1:0]       rd_0_addr,
    output reg                            rd_0_req,
	
	//To Loger	
	output  reg [EVENT_SIZE-1:0]			event_data,
	output 	reg						event_req,
	
	//Timings
	input                              reset,
    input                              clk,
    input  [MAIN_TIME_BITS-1:0]      time_stamp,
	input 							slot_no
	);
   
//-----------------------------buffer and MAC driving-------------

/*
wire bfr_nearly_empty;
reg [DATA_WIDTH+CTRL_WIDTH-1:0]	bfr_din;   
reg						bfr_wr_en;

   
reg [1:0] q_state;   
sender_fifo
	#(.WIDTH(DATA_WIDTH+CTRL_WIDTH),
	  .MAX_DEPTH_BITS(3),
	  .NEARLY_FULL(2))
	sender_fifo
	(   
	 .din				(bfr_din),
	 .wr_en				(bfr_wr_en),
	 .nearly_empty		(bfr_nearly_empty),

	 .rd_en2				(out_rdy_mac), 
	 .dout    			(out_data_mac),
	 .not_empty			(out_wr_mac),
	 
	 .reset				(reset),
	 .clk				(clk)
	);
   
*/
//-------------------------------------------------------------------------------------
reg [INTER_MOD_WIDTH-1:0] in_data_reg;
reg [1:0] read_state;
reg [1:0] read_state_next;
reg a;
reg b;

reg [SRAM_ADDR_WIDTH-1:0]   rd_0_addr_next;

parameter IDLE = 0;
parameter START_READ = 1;
parameter READ =2 ;


parameter not_in_packet = 0;
parameter in_header = 1;
parameter in_packet = 2;
parameter waiting_4_slot_change = 3;
reg [1:0] in_packet_state;
//------------------------reading SRAM------------------------------


reg rd_0_req_next;
always @(posedge clk) begin
if(reset) begin
	read_state   <= IDLE;
	rd_0_req = 0;
end
else begin
	read_state <= read_state_next;
	rd_0_addr = rd_0_addr_next;
	rd_0_req = rd_0_req_next;
	in_data_reg <= in_data_reg;
	if(in_wr) in_data_reg <= in_data;
end
end // always @ (posedge clk)
   
   
   always @(*) begin
      read_state_next   = read_state;
      rd_0_req_next            = 0;
      rd_0_addr_next[SRAM_ADDR_WIDTH-1:SRAM_ADDR_WIDTH-PORT_BITS]=rd_0_addr[SRAM_ADDR_WIDTH-1:SRAM_ADDR_WIDTH-PORT_BITS];
      rd_0_addr_next[SRAM_ADDR_WIDTH-PORT_BITS-1:0]=rd_0_addr[SRAM_ADDR_WIDTH-PORT_BITS-1:0];
	  ///a=(^rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH]) && rd_0_vld ;
	  ///b=(|rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH]) && rd_0_vld;
      case(read_state)
        IDLE: begin

		if(in_wr) begin
             		read_state_next = START_READ;
		end
        end
	START_READ: begin
		///a=0;
		///b=0;
		read_state_next = READ;
		rd_0_addr_next =  in_data_reg [INTER_MOD_WIDTH-1:INTER_MOD_WIDTH-SRAM_ADDR_WIDTH] ;
        end
	READ: begin

		 ///if( a && b) begin
		if( in_packet_state == waiting_4_slot_change) begin
			read_state_next = IDLE;
       		end 
     	rd_0_req_next = 1;

			rd_0_addr_next[SRAM_ADDR_WIDTH-PORT_BITS-1:0]=rd_0_addr[SRAM_ADDR_WIDTH-PORT_BITS-1:0]+rd_0_ack;

        end

        default: begin end

      endcase 
   end


//--------------------------------------------------------------------------------
reg slot_no_pre;

reg counter;
reg [ROUTE_LENGTH-1:0] flow; 
reg [PACKET_ID_LENGTH-1:0] id;



//wire aa;
wire bb;
wire cc;
//assign aa = ^rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH];
assign bb = |rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH];
assign cc = (rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH] === 'hff);
reg [7:0] payload_counter;

always @(posedge clk) begin
if (reset) begin
	out_data_mac <= 0;
	out_wr_mac <= 0;
	in_packet_state <= not_in_packet;
	counter <= 0;
	event_req <= 0;
	payload_counter <= 0;
end
else begin
	
	
	out_data_mac <= out_data_mac;
	event_req <= 0;
	out_wr_mac <= 0;
	case (in_packet_state)
	not_in_packet:begin
		out_wr_mac <= 0;
		payload_counter <= 0;
		if(in_wr) begin
			in_packet_state <= in_header;
			counter <= 0;
		end
	
	end
	
	in_header:begin
		out_wr_mac <= 0;
		
		if(rd_0_vld & !counter) out_data_mac <= rd_0_data;
		else begin if (rd_0_vld & counter) begin
			out_data_mac[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH] <= rd_0_data[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH];
			out_data_mac[DATA_WIDTH-1:DATA_WIDTH-ROUTE_LENGTH] <= rd_0_data[DATA_WIDTH-1:DATA_WIDTH-ROUTE_LENGTH]-1;
			out_data_mac[DATA_WIDTH-ROUTE_LENGTH-1:DATA_WIDTH-ROUTE_LENGTH-LAG_LENGTH] <= in_data_reg [LAG_LENGTH:1];
			out_data_mac[DATA_WIDTH-ROUTE_LENGTH-LAG_LENGTH-1:0] <= rd_0_data[DATA_WIDTH-ROUTE_LENGTH-LAG_LENGTH-1:0];
		end
		end
		
		if (rd_0_vld & cc) begin
			out_wr_mac <= 1;
			counter <= 1;
		end
		if(rd_0_vld & counter) begin
			out_wr_mac <= 1;
			//counter <= 1;
		end
		
		if (counter) begin
			in_packet_state <= in_packet;
			flow <= rd_0_data[DATA_WIDTH-1:DATA_WIDTH-ROUTE_LENGTH];
			id <= rd_0_data[PACKET_ID_LENGTH-1:0];
		end
			
	end
	
	in_packet:begin
	 
	 /*
		if(rd_0_vld) begin
			out_data_mac <= rd_0_data;
			if (payload_counter<'hbb) begin
				out_wr_mac <= 1;
				payload_counter <= payload_counter +1;
			end
			else begin 
				out_data_mac[71:64] <='h10;
				out_wr_mac <= 0;
				in_packet_state <= waiting_4_slot_change;
			end
		end*/
		if(rd_0_vld) begin
			out_data_mac <= rd_0_data;
			
			if (!(bb)) begin //aa&bb)) begin 
				out_wr_mac <= 1;
				out_data_mac[CTRL_WIDTH+DATA_WIDTH-1:DATA_WIDTH] <= 0;
			end
			else out_wr_mac <= 0;
		end
		
		if (bb & rd_0_vld) begin//aa & bb & rd_0_vld) begin
			in_packet_state <= waiting_4_slot_change;
		end 
			
	end
	waiting_4_slot_change:begin
		if(slot_no != slot_no_pre) begin
			in_packet_state <= not_in_packet;
			out_wr_mac <= 1;
			event_req <= 1;
		end
	end
	
	default: begin end
	endcase
	
end
end


always @(posedge clk) begin		
	if (reset) slot_no_pre <= 0;
	else begin 
		slot_no_pre <= slot_no;
	end
end


//----------------------------send to log--------

always @(posedge clk)
begin
  event_data <= {id, time_stamp, in_data_reg [INTER_MOD_WIDTH-1:INTER_MOD_WIDTH-SRAM_ADDR_WIDTH], flow, in_data_reg [LAG_LENGTH:1], PORT_TYPE};
  
end

endmodule 
