///////////////////////////////////////////////////////////////////////////////
//
// Module: my_small_fifo.v
// Project: BJP1
// Description: 
//				It's a multi-propose FIFO, for getting data from previous module,
//				writing and reading to SRAM and etc.
//
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

  module sram_arbiter_fifo
    #(parameter WIDTH = 20,
      parameter MAX_DEPTH_BITS = 3,
      parameter NEARLY_FULL = 2**MAX_DEPTH_BITS - 1)
    (
	 // Data IN	   
     input [WIDTH-1:0] din,
     input          wr_en,
     input          rd_en,
     // Data OUT
     output reg [WIDTH-1:0]  dout,
     output         nearly_full,
     output         not_empty,
     //Timing
     input          reset,
     input          clk
     );


parameter MAX_DEPTH        = 2 ** MAX_DEPTH_BITS;
   
reg [WIDTH-1:0] queue [MAX_DEPTH - 1 : 0];
reg [MAX_DEPTH_BITS - 1 : 0] rd_ptr;
reg [MAX_DEPTH_BITS - 1 : 0] wr_ptr;
reg [MAX_DEPTH_BITS  : 0] depth;
reg depth0;

always @(posedge clk) begin
depth0 <= depth[0];
end

assign nearly_full = depth >= NEARLY_FULL;
assign not_empty = (|depth[MAX_DEPTH_BITS:1]);

wire not_empty2;
assign not_empty2 = |depth;


always @(posedge clk)
begin
   if (reset) begin
      rd_ptr <= 'h0;
      wr_ptr <= 'h0;
      depth  <= 'h0;
   end
   else begin
	  if (wr_en) queue[wr_ptr] = din;
      if (wr_en) wr_ptr <= wr_ptr + 'h1;
      if (rd_en & not_empty2) rd_ptr <= rd_ptr + 'h1;
      if (wr_en & ~rd_en ) depth <= depth + 'h1;
      else if (~wr_en & rd_en & (depth!=0)) depth <= depth - 'h1;
   end
end


always @(posedge clk)
begin
	if (not_empty)
      dout = queue[rd_ptr+rd_en];
end





endmodule 
