///////////////////////////////////////////////////////////////////////////////
//
// Module: timer.v
// Project: BJP1
// Description: 
//              Generate	time_stamp
//							slot_no
//
//				and also when it is near to change slot number, send a
//				notification on near_to_change_slot for sender block.				
//
// By hamed.ty@gmail.com              
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module timer
	#(
	  parameter TIME_WIDTH=19,
	  parameter SLOT_NO_BITS=8,
	  parameter BITS_4_SLOTTING=11,
	  parameter SLOT_SIZE=1523,
	  parameter NEAR_TO_CHANGE = 1200
	  )
		  
    (// --- data path interface
    output reg [TIME_WIDTH-1:0]      time_stamp,
    output reg [SLOT_NO_BITS-1:0]        slot_no,
    output reg				near_to_change_slot,
	
    input			clk,
	input			reset
   );
reg [BITS_4_SLOTTING-1:0] slotting_timer;
   
   always @(posedge clk) 
   begin
	if (reset) begin
		slot_no <= 0;
		slotting_timer <=0;
		near_to_change_slot <= 0;
	end
	else begin
	   if (slotting_timer == SLOT_SIZE) begin
		   slotting_timer <= 0;
		   slot_no <= slot_no +1;
		   near_to_change_slot <= 0;
	   end
	   else begin
			slotting_timer <= slotting_timer + 1;
	   end
		
	   if (slotting_timer == (SLOT_SIZE-NEAR_TO_CHANGE)) begin
			near_to_change_slot <= 1;
	   end
		
   end
  
end

always @(posedge clk)
begin
	if (reset) begin
		time_stamp <= 0;
	end
	else begin
	   time_stamp <= time_stamp + 1;	
   end
  end
   
endmodule
