`timescale 1ns/1ps


/******************************************************
 * Even numbered ports are IO sinks/sources
 * Odd numbered ports are CPU ports corresponding to
 * IO sinks/sources to rpovide direct access to them
 ******************************************************/
module user_data_path
  #(
	parameter NUM_OUTPUT_QUEUES = 8,
	parameter NUM_INPUT_QUEUES = 8,
	
	parameter DATA_WIDTH = 64,
    parameter CTRL_WIDTH=DATA_WIDTH/8,
    parameter UDP_REG_SRC_WIDTH = 2,
    parameter SRAM_DATA_WIDTH = DATA_WIDTH+CTRL_WIDTH,
    parameter SRAM_ADDR_WIDTH = 19,
	parameter MAIN_TIME_BITS= 19,
	parameter SLOT_WIDTH= 8,
	parameter BITS_4_SLOTTING=11,
	parameter SLOT_SIZE=1523,
	parameter NEAR_TO_CHANGE = 1200,
	parameter NO_OF_PORTS = 4,
	parameter PORT_BITS = 2,
	parameter LAG_LENGTH = 5,
	parameter ROUTE_LENGTH = 4,
	parameter PACKET_ID_LENGTH = 12,
	parameter NORMAL_DELAY = 10, //(slot numbers)
	parameter SCHEDULER_TABLE_SIZE = 32,
	parameter LOG_PACKET_SIZE = 10,
	parameter EVENT_SIZE = PACKET_ID_LENGTH+MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS,
	parameter INTER_MOD_WIDTH = SRAM_ADDR_WIDTH + LAG_LENGTH + 1)

   (
    input  [DATA_WIDTH-1:0]            in_data_0,
    input  [CTRL_WIDTH-1:0]            in_ctrl_0,
    input                              in_wr_0,
    output                             in_rdy_0,

    input  [DATA_WIDTH-1:0]            in_data_1,
    input  [CTRL_WIDTH-1:0]            in_ctrl_1,
    input                              in_wr_1,
    output                             in_rdy_1,

    input  [DATA_WIDTH-1:0]            in_data_2,
    input  [CTRL_WIDTH-1:0]            in_ctrl_2,
    input                              in_wr_2,
    output                             in_rdy_2,

    input  [DATA_WIDTH-1:0]            in_data_3,
    input  [CTRL_WIDTH-1:0]            in_ctrl_3,
    input                              in_wr_3,
    output                             in_rdy_3,

    input  [DATA_WIDTH-1:0]            in_data_4,
    input  [CTRL_WIDTH-1:0]            in_ctrl_4,
    input                              in_wr_4,
    output                             in_rdy_4,

    input  [DATA_WIDTH-1:0]            in_data_5,
    input  [CTRL_WIDTH-1:0]            in_ctrl_5,
    input                              in_wr_5,
    output                             in_rdy_5,

    input  [DATA_WIDTH-1:0]            in_data_6,
    input  [CTRL_WIDTH-1:0]            in_ctrl_6,
    input                              in_wr_6,
    output                             in_rdy_6,

    input  [DATA_WIDTH-1:0]            in_data_7,
    input  [CTRL_WIDTH-1:0]            in_ctrl_7,
    input                              in_wr_7,
    output                             in_rdy_7,


    output  [DATA_WIDTH-1:0]           out_data_0,
    output  [CTRL_WIDTH-1:0]           out_ctrl_0,
    output                             out_wr_0,
    input                              out_rdy_0,

    output  [DATA_WIDTH-1:0]           out_data_1,
    output  [CTRL_WIDTH-1:0]           out_ctrl_1,
    output                             out_wr_1,
    input                              out_rdy_1,

    output  [DATA_WIDTH-1:0]           out_data_2,
    output  [CTRL_WIDTH-1:0]           out_ctrl_2,
    output                             out_wr_2,
    input                              out_rdy_2,

    output  [DATA_WIDTH-1:0]           out_data_3,
    output  [CTRL_WIDTH-1:0]           out_ctrl_3,
    output                             out_wr_3,
    input                              out_rdy_3,

    output  [DATA_WIDTH-1:0]           out_data_4,
    output  [CTRL_WIDTH-1:0]           out_ctrl_4,
    output                             out_wr_4,
    input                              out_rdy_4,

    output  [DATA_WIDTH-1:0]           out_data_5,
    output  [CTRL_WIDTH-1:0]           out_ctrl_5,
    output                             out_wr_5,
    input                              out_rdy_5,

    output  [DATA_WIDTH-1:0]           out_data_6,
    output  [CTRL_WIDTH-1:0]           out_ctrl_6,
    output                             out_wr_6,
    input                              out_rdy_6,

    output  [DATA_WIDTH-1:0]           out_data_7,
    output  [CTRL_WIDTH-1:0]           out_ctrl_7,
    output                             out_wr_7,
    input                              out_rdy_7,


     // interface to SRAM
     output [SRAM_ADDR_WIDTH-1:0]       wr_0_addr,
     output                             wr_0_req,
     input                              wr_0_ack,
     output [SRAM_DATA_WIDTH-1:0]       wr_0_data,
     
     input                              rd_0_ack,
     input  [SRAM_DATA_WIDTH-1:0]       rd_0_data,
     input                              rd_0_vld,
     output [SRAM_ADDR_WIDTH-1:0]       rd_0_addr,
     output                             rd_0_req,


	 input                              rd_1_ack,
	 input  [SRAM_DATA_WIDTH-1:0]       rd_1_data,
	 input                              rd_1_vld,
	 output [SRAM_ADDR_WIDTH-1:0]       rd_1_addr,
	 output                             rd_1_req,

     // interface to DRAM
     /* TBD */

     // register interface
     input                              reg_req,
     output                             reg_ack,
     input                              reg_rd_wr_L,
     input [`UDP_REG_ADDR_WIDTH-1:0]    reg_addr,
     output [`CPCI_NF2_DATA_WIDTH-1:0]  reg_rd_data,
     input [`CPCI_NF2_DATA_WIDTH-1:0]   reg_wr_data,
   
     // misc
     input                              reset,
     input                              clk);

	 
wire in_arb_reg_req;
wire in_arb_reg_rd_wr_L;
wire  [`IN_ARB_REG_ADDR_WIDTH-1:0] in_arb_reg_addr;
wire  [`CPCI_NF2_DATA_WIDTH-1:0] in_arb_reg_wr_data;
wire  [`CPCI_NF2_DATA_WIDTH-1:0]      in_arb_reg_rd_data;
wire in_arb_reg_ack;



//---------------timer-----------
wire timer_near_change_slot;
wire [MAIN_TIME_BITS-1:0] timer_time_stamp;
wire [SLOT_WIDTH-1:0]	timer_time_slot;

wire [NO_OF_PORTS-1:0]			arbitter_scheduler_wr;
wire [INTER_MOD_WIDTH-1:0]		arbitter_scheduler_data;


wire [INTER_MOD_WIDTH-1:0]		scheduler_sender_data0;
wire 							scheduler_sender_wr0;
wire [INTER_MOD_WIDTH-1:0]		scheduler_sender_data1;
wire 							scheduler_sender_wr1;
wire [INTER_MOD_WIDTH-1:0]		scheduler_sender_data2;
wire 							scheduler_sender_wr2;
wire [INTER_MOD_WIDTH-1:0]		scheduler_sender_data3;
wire 							scheduler_sender_wr3;

wire [EVENT_SIZE-1:0] arbitter_logger_data;
wire 		arbitter_logger_req;


wire [EVENT_SIZE-1:0] sender_logger_data0;
wire 		sender_logger_req0;

wire [EVENT_SIZE-1:0] sender_logger_data1;
wire 		sender_logger_req1;
//regs
wire                             in_arb_in_reg_req;
wire                             in_arb_in_reg_ack;
wire                             in_arb_in_reg_rd_wr_L;
wire [`UDP_REG_ADDR_WIDTH-1:0]   in_arb_in_reg_addr;
wire [`CPCI_NF2_DATA_WIDTH-1:0]  in_arb_in_reg_data;
wire [UDP_REG_SRC_WIDTH-1:0]     in_arb_in_reg_src;

wire                             udp_reg_req_in;
wire                             udp_reg_ack_in;
wire                             udp_reg_rd_wr_L_in;
wire [`UDP_REG_ADDR_WIDTH-1:0]   udp_reg_addr_in;
wire [`CPCI_NF2_DATA_WIDTH-1:0]  udp_reg_data_in;
wire [UDP_REG_SRC_WIDTH-1:0]     udp_reg_src_in;
timer
	#(
	  .TIME_WIDTH(MAIN_TIME_BITS),
	  .SLOT_NO_BITS(SLOT_WIDTH),
	  .BITS_4_SLOTTING(BITS_4_SLOTTING),
	  .SLOT_SIZE(SLOT_SIZE),
	  .NEAR_TO_CHANGE(NEAR_TO_CHANGE)	  
	  )
	 timer 
	(// --- data path interface
    .time_stamp (timer_time_stamp),
    .slot_no (timer_time_slot),
	.near_to_change_slot (timer_near_change_slot),
	
    .clk	(clk),
	.reset	(reset)
   );

//-------------Input_arbitter ---------------

my_input_arbiter
#(.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.MAIN_TIME_BITS(MAIN_TIME_BITS),
.SRAM_DATA_WIDTH(SRAM_DATA_WIDTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.UDP_REG_SRC_WIDTH(UDP_REG_SRC_WIDTH),
.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.INTER_MOD_WIDTH(SRAM_ADDR_WIDTH + LAG_LENGTH + 1),
.PACKET_ID_LENGTH(PACKET_ID_LENGTH),
.NO_OF_PORTS(NO_OF_PORTS),
.PORT_BITS(PORT_BITS),
.EVENT_SIZE(PACKET_ID_LENGTH+MAIN_TIME_BITS+SRAM_ADDR_WIDTH+LAG_LENGTH+ROUTE_LENGTH+PORT_BITS)
)
my_input_arbiter
   (
    // From MACs
    .in_data_0(in_data_0),
    .in_ctrl_0(in_ctrl_0),
    .in_wr_0(in_wr_0),
    .in_rdy_0(in_rdy_0),

    .in_data_2(in_data_2),
    .in_ctrl_2(in_ctrl_2),
    .in_wr_2(in_wr_2),
    .in_rdy_2(in_rdy_2),
	
    .in_data_4(in_data_4),
    .in_ctrl_4(in_ctrl_4),
    .in_wr_4(in_wr_4),
    .in_rdy_4(in_rdy_4),
	
	.in_data_6(in_data_6),
    .in_ctrl_6(in_ctrl_6),
    .in_wr_6(in_wr_6),
    .in_rdy_6(in_rdy_6),
	// To Scheduler
    .sch(arbitter_scheduler_data),
    .sch_wr(arbitter_scheduler_wr),

	
	// To Sram
	.wr_0_addr(wr_0_addr),
    .wr_0_data(wr_0_data),
    .wr_0_req(wr_0_req),
    .wr_0_ack(wr_0_ack),
	
	// To Logger
	.event0(arbitter_logger_data),
	.event_wr(arbitter_logger_req),
	//registers
	.reg_req_in(in_arb_in_reg_req),
	.reg_ack_in(in_arb_in_reg_ack),
	.reg_rd_wr_L_in(in_arb_in_reg_rd_wr_L),
	.reg_addr_in(in_arb_in_reg_addr),
	.reg_data_in(in_arb_in_reg_data),
	.reg_src_in(in_arb_in_reg_src),

	.reg_req_out(udp_reg_req_in),
	.reg_ack_out(udp_reg_ack_in),
	.reg_rd_wr_L_out(udp_reg_rd_wr_L_in),
	.reg_addr_out(udp_reg_addr_in),
	.reg_data_out(udp_reg_data_in),
	.reg_src_out(udp_reg_src_in),
	
    //Timings
    .reset(reset),
    .clk(clk),
	.time_stamp(timer_time_stamp),
    .slot_no(timer_time_slot[0])
	 
   );
 
//---------Scheduler0----------------

scheduler
#(.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.NORMAL_DELAY(NORMAL_DELAY),
.TABLE_SIZE(SCHEDULER_TABLE_SIZE),
.PTR_SIZE(LAG_LENGTH),
.SLOT_NO_BITS(SLOT_WIDTH)
)	
scheduler0
	( 
	// From Input Arbiter
    .in_data(arbitter_scheduler_data),
    .in_wr(arbitter_scheduler_wr[0]),
	
	// To Sender
	.out_data(scheduler_sender_data0),
    .out_wr(scheduler_sender_wr0),
    
	//Timings
	.reset(reset),
    .clk(clk),
    .slot_no(timer_time_slot),
	.near_to_change_slot(timer_near_change_slot)
	);
//---------Scheduler1----------------
scheduler
#(.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.NORMAL_DELAY(NORMAL_DELAY),
.TABLE_SIZE(SCHEDULER_TABLE_SIZE),
.PTR_SIZE(LAG_LENGTH),
.SLOT_NO_BITS(SLOT_WIDTH )
)	
scheduler1
	( 
	// From Input Arbiter
    .in_data(arbitter_scheduler_data),
    .in_wr(arbitter_scheduler_wr[1]),
	
	// To Sender
	.out_data(scheduler_sender_data1),
    .out_wr(scheduler_sender_wr1),
    
	//Timings
	.reset(reset),
    .clk(clk),
    .slot_no(timer_time_slot),
	.near_to_change_slot(timer_near_change_slot)
	);

//---------Scheduler2----------------

scheduler
#(.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.NORMAL_DELAY(NORMAL_DELAY),
.TABLE_SIZE(SCHEDULER_TABLE_SIZE),
.PTR_SIZE(LAG_LENGTH),
.SLOT_NO_BITS(SLOT_WIDTH )
)
scheduler2
	( 
	// From Input Arbiter
    .in_data(arbitter_scheduler_data),
    .in_wr(arbitter_scheduler_wr[2]),
	
	// To Sender
	.out_data(scheduler_sender_data2),
    .out_wr(scheduler_sender_wr2),
    
	//Timings
	.reset(reset),
    .clk(clk),
    .slot_no(timer_time_slot),
	.near_to_change_slot(timer_near_change_slot)
	);
	
//---------Scheduler3----------------

scheduler
#(.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.NORMAL_DELAY(NORMAL_DELAY),
.TABLE_SIZE(SCHEDULER_TABLE_SIZE),
.PTR_SIZE(LAG_LENGTH),
.SLOT_NO_BITS(SLOT_WIDTH )
)	
scheduler3
	( 
	// From Input Arbiter
    .in_data(arbitter_scheduler_data),
    .in_wr(arbitter_scheduler_wr[3]),
	
	// To Sender
	.out_data(scheduler_sender_data3),
    .out_wr(scheduler_sender_wr3),
    
	//Timings
	.reset(reset),
    .clk(clk),
    .slot_no(timer_time_slot),
	.near_to_change_slot(timer_near_change_slot)
	);
//---------------sender0----------

wire [SRAM_DATA_WIDTH-1:0] sram_data;
wire [SRAM_ADDR_WIDTH-1:0] sram_addr0;
wire [SRAM_ADDR_WIDTH-1:0] sram_addr1;
wire [SRAM_ADDR_WIDTH-1:0] sram_addr2;
wire [SRAM_ADDR_WIDTH-1:0] sram_addr3;

wire [NO_OF_PORTS-1:0] sram_wr;
wire [NO_OF_PORTS-1:0] sram_ack;
wire [NO_OF_PORTS-1:0] sram_req;

sender
#(
.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.MAIN_TIME_BITS(MAIN_TIME_BITS),
.SRAM_DATA_WIDTH(SRAM_DATA_WIDTH),
.EVENT_SIZE(EVENT_SIZE),
.PORT_BITS(PORT_BITS),
.PACKET_ID_LENGTH(PACKET_ID_LENGTH),
.PORT_TYPE(2'b00)

)
sender0
	( 
	// From Scheduler
    .in_data(scheduler_sender_data0),
    .in_wr(scheduler_sender_wr0),
	
	// To MAC
    .out_data_mac({out_ctrl_0, out_data_0}),
    .out_wr_mac(out_wr_0),
    .out_rdy_mac(out_rdy_0),
	//From SRAM
    .rd_0_data(rd_0_data),
    .rd_0_vld(rd_0_vld),

	.rd_0_ack(rd_0_ack),
    .rd_0_addr(rd_0_addr),
    .rd_0_req(rd_0_req),
	
	//To Loger	
	.event_data(sender_logger_data0),
	.event_req(sender_logger_req0),
	
	//Timings
	.reset(reset),
    .clk(clk),
    .time_stamp(timer_time_stamp),
	.slot_no(timer_time_slot[0])
	);

//---------------sender1----------


sender
#(
.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.MAIN_TIME_BITS(MAIN_TIME_BITS),
.SRAM_DATA_WIDTH(SRAM_DATA_WIDTH),
.EVENT_SIZE(EVENT_SIZE),
.PORT_BITS(PORT_BITS),
.PACKET_ID_LENGTH(PACKET_ID_LENGTH),
.PORT_TYPE(2'b01)

)
sender1
	( 
	// From Scheduler
    .in_data(scheduler_sender_data1),
    .in_wr(scheduler_sender_wr1),
	
	// To MAC
    .out_data_mac({out_ctrl_2, out_data_2}),
    .out_wr_mac(out_wr_2),
    .out_rdy_mac(out_rdy_2),
	//From SRAM
    .rd_0_data(rd_1_data),
    .rd_0_vld(rd_1_vld),

	.rd_0_ack(rd_1_ack),
    .rd_0_addr(rd_1_addr),
    .rd_0_req(rd_1_req),	
	//To Loger	
	.event_data(sender_logger_data1),
	.event_req(sender_logger_req1),
	
	//Timings
	.reset(reset),
    .clk(clk),
    .time_stamp(timer_time_stamp),
	.slot_no(timer_time_slot[0])
	);
//---------------sender2----------


sender
#(
.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.MAIN_TIME_BITS(MAIN_TIME_BITS),
.SRAM_DATA_WIDTH(SRAM_DATA_WIDTH),
.EVENT_SIZE(EVENT_SIZE),
.PORT_BITS(PORT_BITS),
.PACKET_ID_LENGTH(PACKET_ID_LENGTH),
.PORT_TYPE(2'b10)

)
sender2
	( 
	// From Scheduler
    .in_data(scheduler_sender_data2),
    .in_wr(scheduler_sender_wr2),
	
	// To MAC
    .out_data_mac({out_ctrl_4, out_data_4}),
    .out_wr_mac(out_wr_4),
    .out_rdy_mac(out_rdy_4),
	//From SRAM
    .rd_0_data(sram_data),
    .rd_0_vld(sram_wr[2]),

	.rd_0_ack(sram_ack[2]),
    .rd_0_addr(sram_addr2),
    .rd_0_req(sram_req[2]),
	
	//To Loger	
	.event_data(sender_logger_data2),
	.event_req(sender_logger_req2),
	
	//Timings
	.reset(reset),
    .clk(clk),
    .time_stamp(timer_time_stamp),
	.slot_no(timer_time_slot[0])
	);
//---------------sender3----------


sender
#(
.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.SRAM_ADDR_WIDTH(SRAM_ADDR_WIDTH),
.LAG_LENGTH(LAG_LENGTH),
.ROUTE_LENGTH(ROUTE_LENGTH),
.INTER_MOD_WIDTH(INTER_MOD_WIDTH),
.MAIN_TIME_BITS(MAIN_TIME_BITS),
.SRAM_DATA_WIDTH(SRAM_DATA_WIDTH),
.EVENT_SIZE(EVENT_SIZE),
.PORT_BITS(PORT_BITS),
.PACKET_ID_LENGTH(PACKET_ID_LENGTH),
.PORT_TYPE(2'b11)

)
sender3
	( 
	// From Scheduler
    .in_data(scheduler_sender_data3),
    .in_wr(scheduler_sender_wr3),
	
	// To MAC
    .out_data_mac({out_ctrl_6, out_data_6}),
    .out_wr_mac(out_wr_6),
    .out_rdy_mac(out_rdy_6),
	//From SRAM
    .rd_0_data(sram_data),
    .rd_0_vld(sram_wr[3]),

	.rd_0_ack(sram_ack[3]),
    .rd_0_addr(sram_addr3),
    .rd_0_req(sram_req[3]),
	
	//To Loger	
	.event_data(sender_logger_data3),
	.event_req(sender_logger_req3),
	
	//Timings
	.reset(reset),
    .clk(clk),
    .time_stamp(timer_time_stamp),
	.slot_no(timer_time_slot[0])
	);
//-------------sram_read_arbiter---------------
my_sram_read_arbiter
#( .SENDERS_NO(4)
	)
my_sram_read_arbiter
	( 
	//To SRAM
    .rd_0_data(0),
    .rd_0_vld(1),

	.rd_0_ack(1),
    .rd_0_addr(),
    .rd_0_req(),
	
	//From Senders SRAM
    .data_out(sram_data),
    .wr(sram_wr),
///////////
	.ack(sram_ack),
	.req(sram_req),	
    
	.addr0(sram_addr0),
	.addr1(sram_addr1),
	.addr2(sram_addr2),
	.addr3(sram_addr3),
	
	//Timings
	.reset(reset),
    .clk(clk)

	);

//--------Logger--------------
logger
#(.EVENT_SIZE(EVENT_SIZE),
.DATA_WIDTH(DATA_WIDTH),
.CTRL_WIDTH(CTRL_WIDTH),
.PACKET_SIZE(LOG_PACKET_SIZE)
)
logger
  ( 
  //From arbitter
	.data_arbitter		(arbitter_logger_data),
	.req_arbitter		(arbitter_logger_req),

	//From sender
	.data_sender0		(sender_logger_data0),
	.req_sender0			(sender_logger_req0),

	.data_sender1		(sender_logger_data1),
	.req_sender1			(sender_logger_req1),
	
	.data_sender2		(sender_logger_data2),
	.req_sender2			(sender_logger_req2),
	
	.data_sender3		(sender_logger_data3),
	.req_sender3			(sender_logger_req3),
	// To CPU 1
    .out_data_mac	(out_data_1),
	.out_ctrl_mac	(out_ctrl_1), 
    .out_wr_mac		(out_wr_1),
    .out_rdy_mac	(out_rdy_1),
	
	//Timings
    .clk	(clk),
	.reset	(reset)
	);
	
//--------------------REGS-----------------

   udp_reg_master #(
      .UDP_REG_SRC_WIDTH (UDP_REG_SRC_WIDTH)
   ) udp_reg_master (
      // Core register interface signals
      .core_reg_req                          (reg_req),
      .core_reg_ack                          (reg_ack),
      .core_reg_rd_wr_L                      (reg_rd_wr_L),

      .core_reg_addr                         (reg_addr),

      .core_reg_rd_data                      (reg_rd_data),
      .core_reg_wr_data                      (reg_wr_data),

      // UDP register interface signals (output)
      .reg_req_out                           (in_arb_in_reg_req),
      .reg_ack_out                           (in_arb_in_reg_ack),
      .reg_rd_wr_L_out                       (in_arb_in_reg_rd_wr_L),

      .reg_addr_out                          (in_arb_in_reg_addr),
      .reg_data_out                          (in_arb_in_reg_data),

      .reg_src_out                           (in_arb_in_reg_src),

      // UDP register interface signals (input)
      .reg_req_in                            (udp_reg_req_in),
      .reg_ack_in                            (udp_reg_ack_in),
      .reg_rd_wr_L_in                        (udp_reg_rd_wr_L_in),

      .reg_addr_in                           (udp_reg_addr_in),
      .reg_data_in                           (udp_reg_data_in),

      .reg_src_in                            (udp_reg_src_in),

      //
      .clk                                   (clk),
      .reset                                 (reset)
   );	


endmodule // user_data_path
